@@ -48,29 +48,28 @@ impl QuantizedArray {
     where
         R: Read + Seek,
     {
-        let projection = read
-            .read_u32::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read quantized embedding matrix projection", e))?
-            != 0;
+        let projection = read.read_u32::<LittleEndian>().map_err(|e| {
+            Error::read_error("Cannot read quantized embedding matrix projection", e)
+        })? != 0;
         let read_norms = read
             .read_u32::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read quantized embedding matrix norms", e))?
+            .map_err(|e| Error::read_error("Cannot read quantized embedding matrix norms", e))?
             != 0;
         let quantized_len = read
             .read_u32::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read quantized embedding length", e))?
+            .map_err(|e| Error::read_error("Cannot read quantized embedding length", e))?
             as usize;
         let reconstructed_len = read
             .read_u32::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read reconstructed embedding length", e))?
+            .map_err(|e| Error::read_error("Cannot read reconstructed embedding length", e))?
             as usize;
         let n_centroids = read
             .read_u32::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read number of subquantizers", e))?
+            .map_err(|e| Error::read_error("Cannot read number of subquantizers", e))?
             as usize;
         let n_embeddings = read
             .read_u64::<LittleEndian>()
-            .map_err(|e| Error::io_error("Cannot read number of quantized embeddings", e))?
+            .map_err(|e| Error::read_error("Cannot read number of quantized embeddings", e))?
             .try_into()
             .map_err(|_| Error::Overflow)?;
 
@@ -84,15 +83,15 @@ impl QuantizedArray {
 
         let n_padding =
             padding::<f32>(read.seek(SeekFrom::Current(0)).map_err(|e| {
-                Error::io_error("Cannot get file position for computing padding", e)
+                Error::read_error("Cannot get file position for computing padding", e)
             })?);
         read.seek(SeekFrom::Current(n_padding as i64))
-            .map_err(|e| Error::io_error("Cannot skip padding", e))?;
+            .map_err(|e| Error::read_error("Cannot skip padding", e))?;
 
         let projection = if projection {
             let mut projection = Array2::zeros((reconstructed_len, reconstructed_len));
             read.read_f32_into::<LittleEndian>(projection.as_slice_mut().unwrap())
-                .map_err(|e| Error::io_error("Cannot read projection matrix", e))?;
+                .map_err(|e| Error::read_error("Cannot read projection matrix", e))?;
             Some(projection)
         } else {
             None
@@ -106,7 +105,7 @@ impl QuantizedArray {
             .into_dimension();
         let mut quantizers = Array::zeros(quantizer_shape);
         read.read_f32_into::<LittleEndian>(quantizers.as_slice_mut().unwrap())
-            .map_err(|e| Error::io_error("Cannot read subquantizer", e))?;
+            .map_err(|e| Error::read_error("Cannot read subquantizer", e))?;
 
         Ok(PqRead {
             n_embeddings,
@@ -127,7 +126,7 @@ impl QuantizedArray {
         write
             .write_u32::<LittleEndian>(ChunkIdentifier::QuantizedArray as u32)
             .map_err(|e| {
-                Error::io_error(
+                Error::write_error(
                     "Cannot write quantized embedding matrix chunk identifier",
                     e,
                 )
@@ -137,10 +136,9 @@ impl QuantizedArray {
         // reconstructed_len (u32), n_centroids (u32), rows (u64),
         // types (2 x u32 bytes), padding, projection matrix,
         // centroids, norms, quantized data.
-        let n_padding =
-            padding::<f32>(write.seek(SeekFrom::Current(0)).map_err(|e| {
-                Error::io_error("Cannot get file position for computing padding", e)
-            })?);
+        let n_padding = padding::<f32>(write.seek(SeekFrom::Current(0)).map_err(|e| {
+            Error::write_error("Cannot get file position for computing padding", e)
+        })?);
         let chunk_size = size_of::<u32>()
             + size_of::<u32>()
             + size_of::<u32>()
@@ -163,51 +161,53 @@ impl QuantizedArray {
         write
             .write_u64::<LittleEndian>(chunk_size as u64)
             .map_err(|e| {
-                Error::io_error("Cannot write quantized embedding matrix chunk length", e)
+                Error::write_error("Cannot write quantized embedding matrix chunk length", e)
             })?;
 
         write
             .write_u32::<LittleEndian>(quantizer.projection().is_some() as u32)
             .map_err(|e| {
-                Error::io_error("Cannot write quantized embedding matrix projection", e)
+                Error::write_error("Cannot write quantized embedding matrix projection", e)
             })?;
         write
             .write_u32::<LittleEndian>(norms.is_some() as u32)
-            .map_err(|e| Error::io_error("Cannot write quantized embedding matrix norms", e))?;
+            .map_err(|e| Error::write_error("Cannot write quantized embedding matrix norms", e))?;
         write
             .write_u32::<LittleEndian>(quantizer.quantized_len() as u32)
-            .map_err(|e| Error::io_error("Cannot write quantized embedding length", e))?;
+            .map_err(|e| Error::write_error("Cannot write quantized embedding length", e))?;
         write
             .write_u32::<LittleEndian>(quantizer.reconstructed_len() as u32)
-            .map_err(|e| Error::io_error("Cannot write reconstructed embedding length", e))?;
+            .map_err(|e| Error::write_error("Cannot write reconstructed embedding length", e))?;
         write
             .write_u32::<LittleEndian>(quantizer.n_quantizer_centroids() as u32)
-            .map_err(|e| Error::io_error("Cannot write number of subquantizers", e))?;
+            .map_err(|e| Error::write_error("Cannot write number of subquantizers", e))?;
         write
             .write_u64::<LittleEndian>(quantized.nrows() as u64)
-            .map_err(|e| Error::io_error("Cannot write number of quantized embeddings", e))?;
+            .map_err(|e| Error::write_error("Cannot write number of quantized embeddings", e))?;
 
         // Quantized and reconstruction types.
         write
             .write_u32::<LittleEndian>(u8::type_id())
-            .map_err(|e| Error::io_error("Cannot write quantized embedding type identifier", e))?;
+            .map_err(|e| {
+                Error::write_error("Cannot write quantized embedding type identifier", e)
+            })?;
         write
             .write_u32::<LittleEndian>(f32::type_id())
             .map_err(|e| {
-                Error::io_error("Cannot write reconstructed embedding type identifier", e)
+                Error::write_error("Cannot write reconstructed embedding type identifier", e)
             })?;
 
         let padding = vec![0u8; n_padding as usize];
         write
             .write_all(&padding)
-            .map_err(|e| Error::io_error("Cannot write padding", e))?;
+            .map_err(|e| Error::write_error("Cannot write padding", e))?;
 
         // Write projection matrix.
         if let Some(projection) = quantizer.projection() {
             for row in projection.outer_iter() {
                 for &col in row {
                     write.write_f32::<LittleEndian>(col).map_err(|e| {
-                        Error::io_error("Cannot write projection matrix component", e)
+                        Error::write_error("Cannot write projection matrix component", e)
                     })?;
                 }
             }
@@ -217,9 +217,9 @@ impl QuantizedArray {
         for subquantizer in quantizer.subquantizers().outer_iter() {
             for row in subquantizer.outer_iter() {
                 for &col in row {
-                    write
-                        .write_f32::<LittleEndian>(col)
-                        .map_err(|e| Error::io_error("Cannot write subquantizer component", e))?;
+                    write.write_f32::<LittleEndian>(col).map_err(|e| {
+                        Error::write_error("Cannot write subquantizer component", e)
+                    })?;
                 }
             }
         }
@@ -230,7 +230,7 @@ impl QuantizedArray {
                 for &col in row {
                     write
                         .write_f32::<LittleEndian>(col)
-                        .map_err(|e| Error::io_error("Cannot write norm vector component", e))?;
+                        .map_err(|e| Error::write_error("Cannot write norm vector component", e))?;
                 }
             }
         }
@@ -239,7 +239,7 @@ impl QuantizedArray {
         for row in quantized.outer_iter() {
             for &col in row {
                 write.write_u8(col).map_err(|e| {
-                    Error::io_error("Cannot write quantized embedding matrix component", e)
+                    Error::write_error("Cannot write quantized embedding matrix component", e)
                 })?;
             }
         }
@@ -306,7 +306,7 @@ impl ReadChunk for QuantizedArray {
 
         // Read and discard chunk length.
         read.read_u64::<LittleEndian>().map_err(|e| {
-            Error::io_error("Cannot read quantized embedding matrix chunk length", e)
+            Error::read_error("Cannot read quantized embedding matrix chunk length", e)
         })?;
 
         let PqRead {
@@ -318,7 +318,7 @@ impl ReadChunk for QuantizedArray {
         let norms = if read_norms {
             let mut norms = Array1::zeros((n_embeddings,));
             read.read_f32_into::<LittleEndian>(norms.as_slice_mut().unwrap())
-                .map_err(|e| Error::io_error("Cannot read norms", e))?;
+                .map_err(|e| Error::read_error("Cannot read norms", e))?;
             Some(norms)
         } else {
             None
@@ -326,7 +326,7 @@ impl ReadChunk for QuantizedArray {
 
         let mut quantized_embeddings = Array2::zeros((n_embeddings, quantizer.quantized_len()));
         read.read_exact(quantized_embeddings.as_slice_mut().unwrap())
-            .map_err(|e| Error::io_error("Cannot read quantized embeddings", e))?;
+            .map_err(|e| Error::read_error("Cannot read quantized embeddings", e))?;
 
         Ok(QuantizedArray {
             quantizer,
@@ -427,7 +427,7 @@ where
         // upper bound than reductive.
         let max_subquantizer_bits = std::cmp::min(8, (self.shape().0 as f64).log2().trunc() as u32);
         if n_subquantizer_bits > max_subquantizer_bits {
-            return Err(Error::QuantizationError(
+            return Err(Error::Quantization(
                 ReductiveError::IncorrectNSubquantizerBits {
                     max_subquantizer_bits,
                 },
@@ -453,7 +453,7 @@ where
             embeds.view(),
             &mut rng,
         )
-        .map_err(Error::QuantizationError)?;
+        .map_err(Error::Quantization)?;
 
         let quantized_embeddings = quantizer.quantize_batch(embeds.view());
 
@@ -532,7 +532,7 @@ mod mmap {
             quantized_len: usize,
         ) -> Result<Mmap> {
             let offset = read.seek(SeekFrom::Current(0)).map_err(|e| {
-                Error::io_error(
+                Error::read_error(
                     "Cannot get file position for memory mapping embedding matrix",
                     e,
                 )
@@ -545,13 +545,13 @@ mod mmap {
                     .len(matrix_len)
                     .map(&*read.get_ref())
                     .map_err(|e| {
-                        Error::io_error("Cannot memory map quantized embedding matrix", e)
+                        Error::read_error("Cannot memory map quantized embedding matrix", e)
                     })?
             };
 
             // Position the reader after the matrix.
             read.seek(SeekFrom::Current(matrix_len as i64))
-                .map_err(|e| Error::io_error("Cannot skip quantized embedding matrix", e))?;
+                .map_err(|e| Error::read_error("Cannot skip quantized embedding matrix", e))?;
 
             Ok(quantized)
         }
@@ -620,7 +620,7 @@ mod mmap {
 
             // Read and discard chunk length.
             read.read_u64::<LittleEndian>().map_err(|e| {
-                Error::io_error("Cannot read quantized embedding matrix chunk length", e)
+                Error::read_error("Cannot read quantized embedding matrix chunk length", e)
             })?;
 
             let PqRead {
@@ -632,7 +632,7 @@ mod mmap {
             let norms = if read_norms {
                 let mut norms_vec = Array1::zeros((n_embeddings,));
                 read.read_f32_into::<LittleEndian>(norms_vec.as_slice_mut().unwrap())
-                    .map_err(|e| Error::io_error("Cannot read norms", e))?;
+                    .map_err(|e| Error::read_error("Cannot read norms", e))?;
                 Some(norms_vec)
             } else {
                 None
